'Filename   ILI9341-Master_100.bas
'Version    1.00
'Date       2019-03-26
'Purpose    A set of routines to control a graphic LCD-controller via I2C
'           - GLCD_Clear         Clear screen and set background color
'           - GLCD_Text          Show text
'           - GLCD_line          Show line
'           - GLCD_Box           Show box
'           - GLCD_circle        Show circle
'           - GLCD_fill_circle   Show filled circle
'           Test configuration:
'           - Master board
'             ATmega16, Crystal 16 MHz
'             2x16 Text LCD @ PortD.2...D.7
'             I2C is used to send the GLCD commands to the slave board
'             SCL = PortC.0 (TWI hardware ports)
'             SDA = PortC.1
'             PC.2 is input for LCD busy signal (=high if LCD is busy)
'           - Slave board
'             Arduino UNO, ATmega328p, 16 MHz
'             Graphic ILI9341 TFT shield
'           I2C protocoll:
'           Byte 0: bytGD_Addr         Slave address
'           Byte 1: bytGD_Len          Number of data bytes from byte 1 to n
'           Byte 2: bytGD_Cmd          GLCD command
'           Byte 3: bytGD_Dat(1        First data byte
'           ...
'           Byte n: bytGD_Dat(n-2)     Last data byte
'
'Author     Georg Latzel, DL6GL
'Copyright  (c) Georg Latzel, free for private use under GNU license
'Hardware   ATmega 32,  2x16 LCD
'Compiler   BASCOM-AVR 2.0.7.6
'           For readable "camel case" variables mark "Don't change case"
'           in BASCOM Options-Programmer-Environment-Editor
'Edit history
'V1.00      2019-03-25, first release
'           Flash used: 10,540 bytes (64%)
'==============================================================================

'FUSE-Bits (AVR Studio 4.0-Settings, [ ]=no check, [x]=check)
'In ATMEGA-Data sheets: Bit = 1 -> not set = no check
'                       Bit = 0 -> set     = check
'OCDEN      [ ]
'JTAGEN     [ ]
'SPIEN      [x] Do not change!
'EESAVE     [ ]
'BOOTSZ     Boot Flash size=128 words (minimal boot size)
'BOOTRST    [ ]
'CKOPT      [ ]
'BODLEVEL   Brown-out detection at VCC=4.0V
'BODEN      [ ]
'SUT_CKSEL  Ext. Crystal Osc. High Freq., Start-up time 16k + 64ms
'High:      0xDF
'Low:       0x7F

$regfile = "m16def.dat"                           'ATMega16
$crystal = 16000000                               '16 MHz xtal
$hwstack = 60
$swstack = 100
$framesize = 100

$Lib "i2c_twi.lbx"                                'Use hardware TWI

Const strVers = "V1.00"                           'Version number

'Configure LCD on Port D ------------------------------------------------------
Config Lcd = 16 * 2
Config Lcdpin = Pin , Db7 = PortD.7 , Db6 = PortD.6 , Db5 = PortD.5
Config Lcdpin = Pin , Db4 = PortD.4 , E = PortD.3 , Rs = PortD.2
Config Lcdbus = 4

'Configure I2C on dedicated controller SCL/SDA hardware ports -----------------
Config SCL = PortC.0                              'I2C, SCL
Config SDA = PortC.1                              'I2C, SDA
Config TWI = 100000                               'Bus speed 100kHz
DDRC.2 = 0                                        'C.2 = Input
LCD_Busy Alias PInC.2                             'Busy signal from slave

'Variables TWI-Communication  -------------------------------------------------
Const bytGD_Addr = &H74                           'GLCD controller slave address
Dim bytTWI_Status As Byte                         'TWI status
Dim bytGD_Len As Byte                             'Length of data message
Dim bytGD_Cmd As Byte                             'GLCD command

'Common Variables, multiply used ----------------------------------------------
Dim bytTmp0 As Byte
Dim bytTmp1 As Byte
Dim bytHi As Byte
Dim bytLo As Byte
Dim bytLen As Byte
Dim wrdTmp0 As Word
Const TextLen = 50
Dim strText As String * TextLen
Dim strChar1 As String * 1

'For TRX display test
Dim wrdBarColor As Word
Dim wrdBarStart As Word
Dim wrdBarEnd As Word
Dim wrdBarLen As Word
Dim wrdBarMax As Word
Dim dwdFreq As DWord
Dim wrdPwrFrameLen As Word
Dim wrdPwrFrameStart As Word
Dim wrdPwr As Word
Dim sngTmp0 As Single
Dim sngMaxPwr As Single
Dim sngPwr As Single
Dim strMHz As String * 6
Dim strHz As String * 3

'Some colors, more colors are implemented in the slave software
Dim wrdBackColor As Word
Const Transparent = &H1000
Const Black = &H0000
Const Blue = &H001F
Const Green = &H07E0
Const Greenyellow = &HAFE5
Const Red = &HF800
Const White = &HFFFF
Const Yellow = &HFFE0

'Subroutines declarations -----------------------------------------------------
'The GLCD call parameter list has the structure of the BASCOM commands
Declare Function CheckI2C(byVal bytAddr As Byte) As Byte
Declare Sub IsLCDBusy()
Declare Sub GLCD_INIT()
Declare Sub GLCD_Clear(byVal wrdColor As Word)
Declare Sub GLCD_Text(byval strText As String , Byval wrdPosX As Word , Byval wrdPosY As Word , Byval bytFont As Byte , Byval wrdForecolor As Word , Byval wrdBackcolor As Word , Byval bytBold As Byte)
Declare Sub GLCD_Box(byval wrdPosX As Word , Byval wrdPosY As Word , Byval wrdLenX As Word , Byval wrdLenY As Word , Byval bytFill As Byte , Byval wrdBackColor As Word , Byval wrdBordercolor As Word)
Declare Sub GLCD_circle(byVal wrdPosX As Word , byVal wrdPosY As Word , byVal wrdRadius As Word , byVal wrdBordercolor As Word)
Declare Sub GLCD_fill_circle(byVal wrdPosX As Word , byVal wrdPosY As Word , byVal wrdRadius As Word , byVal wrdBackcolor As Word)
Declare Sub GLCD_line(byVal wrdPosX1 As Word , byVal wrdPosY1 As Word , byVal wrdPosX2 As Word , byVal wrdPosY2 As Word , byVal bytWidth As Byte , byVal wrdLineColor As Word)

'Only for testing with the main program
Declare Sub ClearLCDLine(byVal bytLine As Byte)
Declare Sub PaintBar

I2CInit                                           'Initialize I2C bus

'Main program for testing *****************************************************
Cls
Cursor Off
Locate 1 , 1
LCD "ILI9341-Master"
Locate 2 , 1
Lcd strVers
Wait 1

Call ClearLCDLIne(1)
LCD "Check I2C @ " ; hex(bytGD_Addr)
Call ClearLCDLIne(2)
Select Case CheckI2C(bytGD_Addr)                  'Check I2C @ bytGD_Addr
   Case 0
      LCD "Slave NOT OK"
   Case 1
      LCD "Slave is OK "
   Case 2
      LCD "Start error "
End Select
Locate 2 , 14
LCD hex(bytTWI_Status)
Wait 2
CLS

wrdBackColor = Black

Locate 1 , 1
LCD "Send to Slave:"

Do

   'Clear screen, call:
   'Call GLCD_Clear(wrdBackColor)
   Call ClearLCDLine(2)
   LCD "Clear " ; hex(wrdBackColor)
   Call GLCD_Clear(wrdBackColor)
   Wait 1

   'Show Text, call:
   'GLCD_Text(strText , wrdXPos , wrdYpos , bytFont , wrdForecolor , wrdBackcolor , bytBold)
   'Delete a Text from screen: Repeat the GLCD_Text command with Forecolor= Backcolor
   Call GLCD_Text( "Welcome to DL6GL TFT test" , 10 , 10 , 1 , Yellow , wrdBackColor , 2)
   Call GLCD_Text( "6x8 Bold 1 (smallest font)" , 10 , 40 , 1 , Green , Black , 1)
   Call GLCD_Text( "12x16 Bold 1" , 10 , 60 , 2 , Green , Black , 1)
   Call GLCD_Text( "12x16 Bold 2" , 10 , 100 , 2 , Green , Black , 2)
   Wait 3


   'Show boxes, call:
   'GLCD_Box(wrdPosX, wrdPosY, wrdLenX, wrdLenY, bytFill, wrdBackColor, wrdBordercolor)
   Call ClearLCDLine(2)
   LCD "Box"
   Call GLCD_Clear(Black)
   Locate 2 , 5
   LCD "red/blue    "
   Call GLCD_Box(10 , 10 , 300 , 220 , 1 , Red , Blue)
   Locate 2 , 5
   LCD "blue/yellow "
   Call GLCD_Box(30 , 30 , 260 , 180 , 1 , Blue , Yellow)
   Locate 2 , 5
   LCD "green/white "
   Call GLCD_Box(50 , 50 , 220 , 140 , 1 , Green , White)
   Locate 2 , 5
   LCD "red/blue    "
   Call GLCD_Box(70 , 70 , 180 , 100 , 1 , Red , Blue)
   Locate 2 , 5
   LCD "yellow/green"
   Call GLCD_Box(90 , 90 , 140 , 60 , 1 , Yellow , Green)
   Wait 2

   'Show circles, call:
   'GLCD_circle(wrdPosX , wrdPosY , wrdRadius , wrdLineColor)
   Call GLCD_clear(White)
   Call ClearLCDLine(2)
   LCD "Circle"
   Call GLCD_text( "Circles" , 10 , 10 , 2 , Blue , White , 1)
   Call GLCD_circle(160 , 120 , 20 , Red)
   For wrdTmp0 = 20 To 100 Step 5
      Call GLCD_circle(160 , 120 , wrdTmp0 , Red)
   Next
   Wait 2

   'Show filled circles, call:
   'GLCD_circle(wrdPosX , wrdPosY , wrdRadius , wrdBackColor)
   Call GLCD_clear(White)
   Call ClearLCDLine(2)
   LCD "Filled Circle"
   Call GLCD_text( "One filled circle" , 10 , 10 , 2 , Blue , White , 1)
   Call GLCD_fill_circle(60 , 140 , 20 , Red)
   Call GLCD_fill_circle(160 , 140 , 20 , Blue)
   Call GLCD_fill_circle(260 , 140 , 20 , Green)
   Wait 2

   Call GLCD_clear(Black)
   Call GLCD_text( "More filled circles" , 10 , 10 , 2 , Blue , White , 1)
   Call GLCD_fill_circle(160 , 120 , 20 , Red)
   For wrdTmp0 = 20 To 100 Step 5
      Call GLCD_fill_circle(160 , 120 , wrdTmp0 , Yellow)
   Next
   Wait 1

   'Show lines, call:
   'GLCD_line(wrdPosX1 , wrdPosY1 , wrdPosX2 , wrdPosY2 , PenWidth , LineColor)
   'X=0...319
   'Y=0...239
   Call GLCD_clear(Black)
   Call ClearLCDLine(2)
   LCD "Lines"
   Call GLCD_text( "Lines" , 10 , 10 , 2 , Blue , White , 1)
   For wrdTmp0 = 0 To 319 Step 10                 'X
      Call GLCD_line(wrdTmp0 , 0 , wrdTmp0 , 239 , 2 , Red)
   Next wrdTmp0
   For wrdTmp0 = 0 To 239 Step 10                 'Y
      Call GLCD_line(0 , wrdTmp0 , 319 , wrdTmp0 , 2 , Blue)
   Next wrdTmp0
   Wait 1

   'Show TRX display
   Call GLCD_clear(Blue)
   Call ClearLCDLine(2)
   LCD "TRX Display"
   wrdBackColor = Blue
   Call GLCD_text( "TRX 2019 DL6GL" , 150 , 5 , 2 , Yellow , wrdBackColor , 1)
   dwdFreq = 14253456
   strText = str(dwdFreq)
   strMHz = Left(strText , 5)
   strHz = Right(strText , 3)
   strText = Format(strMHz , "0.000")
   Call GLCD_text(strText , 5 , 50 , 2 , White , wrdBackColor , 2)
   strText = strHz + " MHz"
   Call GLCD_text(strText , 155 , 64 , 2 , White , wrdBackColor , 1)
   Call GLCD_text( " TX " , 5 , 100 , 1 , wrdBackColor , White , 3)
   Call GLCD_text( " RX " , 70 , 100 , 1 , White , wrdBackColor , 3)
   Call GLCD_text( " SSB " , 135 , 100 , 1 , wrdBackColor , White , 3)
   Call GLCD_text( " USB " , 225 , 100 , 1 , wrdBackColor , White , 3)
   Waitms 10                                      'Wait before painting box
   'Power bar, red if Pwr > MaxPwr, green else
   wrdPwrFrameLen = 310                           'Power bar frame length (pixel)
   wrdPwrFrameStart = 5                           'Power bar frame start pos. (pixel)
   wrdBarStart = wrdPwrFrameStart + 2             'Power bar start pos (pixel)
   wrdBarMax = wrdPwrFrameLen - 4                 'Net bar length for max power (pixel)
   sngMaxPwr = 100                                'Max power
   wrdBarColor = Green
   Call GLCD_Box(wrdPwrFrameStart , 190 , wrdPwrFrameLen , 20 , 2 , wrdBackColor , White)       'Empty power bar

   'Show power bar
   sngPwr = 48                                    'Current power
   Do                                             'Loop power upwards
      sngPwr = sngPwr + 2
      Call PaintBar
      Waitms 50
   Loop Until sngPwr > 110

   Do                                             'Loop power daownwards
      sngPwr = sngPwr - 2
      Call PaintBar
      Waitms 50
   Loop Until sngPwr < 52
   Wait 1
   wrdBackColor = Black

Loop

'End of main program **********************************************************

'==============================================================================
Sub PaintBar
'Paint power bar

      If sngPwr > sngMaxPwr Then
         wrdBarColor = Red
      Else
         wrdBarColor = Green
      End If
      sngTmp0 = sngPwr / sngMaxPwr
      wrdPwr = sngTmp0 * wrdBarMax                'Pixel for current power
      If wrdPwr > wrdBarMax Then wrdPwr = wrdBarMax

      wrdTmp0 = wrdBarStart                       'Bar start pos.
      wrdBarLen = wrdPwr                          'Bar length
      wrdBarEnd = wrdBarStart + wrdBarLen         'Bar end pos.
      'Show power bar
      Call GLCD_Box(wrdBarStart , 193 , wrdBarLen , 15 , 1 , wrdBarColor , White)
      Waitms 10

      'Show bar with backColor from current bar up to the end
      'i.e. erase upper part of the bar if new bar is shorter than the former bar
      If wrdBarLen < wrdBarMax Then
         wrdTmp0 = wrdBarEnd + 1
         wrdBarEnd = wrdBarStart + wrdBarMax
         wrdBarLen = wrdBarEnd - wrdTmp0
         Call GLCD_Box(wrdTmp0 , 193 , wrdBarLen , 15 , 1 , wrdBackColor , White)
      End If

      strText = Fusing(sngPwr , "#.#")
      strText = strText + "W PEP  "
      Call GLCD_text(strText , 5 , 167 , 2 , White , wrdBackColor , 1)
      wrdTmp0 = sngMaxPwr
      strText = str(wrdTmp0)
      strText = strText + "W"
      Call GLCD_text(strText , 267 , 167 , 2 , White , wrdBackColor , 1)

End Sub

'==============================================================================
Sub IsLCDBusy()
'Check if LCD is still busy (LCD_Busy signal is high)
   Do
      Nop
   Loop Until LCD_Busy = 0

End Sub

'==============================================================================
Function CheckI2C(byVal bytAddr As Byte) As Byte
'Check I2C chip @ address bytAddr
'CheckI2C = 1: OK, 0: else
'Success is checked with TWI status register TWSR
'Status code is stored in the 5 upper bits, to masked out with "And &B11111000"
'Write status code:
'&H08:   Start command was sent
'&H10:   Repeated start command was sent
'&H18:   Slave address (send condition) was sent, ACK received
'&H20:   Slave address (send condition) was sent, NACK received

   I2cstart
      bytTWI_Status = TWSR And &B11111000         'Start status code
      If bytTWI_Status = &H08 Or bytTWI_Status = &H10 Then       'Start condition is set
         I2cwbyte bytAddr                         'Send Slave address
         bytTWI_Status = TWSR And &B11111000      'Slave address write status code
         If bytTWI_Status = &H18 Then             'Slave answered with ACK
            CheckI2C = 1                          'Slave found
         Else
            CheckI2C = 0                          'Slave not found
         End If
      Else
         CheckI2C = 2                             'Start error
      End If
   I2cstop

End Function

'==============================================================================
Sub ClearLCDLine(byVal bytLine As Byte)
'Clear LCD line bytLine
'Only used for Test main program
   Locate bytline , 1
   LCD SPC(16)
   Locate bytline , 1
End Sub

'==============================================================================
Sub GLCD_INIT()
'Send the LCD Init command via I2C to the GLCD controller
'Actually not used. LCD INIT is triggered by Arduino reset in the LCD software
'Init GLCD, GLCD call:
'Sub Lcd_init()

   bytGD_cmd = 1                                  'LCD Init command
   bytGD_Len = 0                                  '0 bytes length message

   Call IsLCDBusy                                 'Wait if LCD is still busy
   'Send it via I2C
   I2CStart
      I2CwByte bytGD_Addr                         'Write I2C address
      I2CwByte bytGD_Len                          'Message length
      I2CwByte bytGD_Cmd                          'Display command
   I2Cstop

End Sub

'==============================================================================
Sub GLCD_Clear(byVal wrdColor As Word)
'Send the Clear screen & set color command via I2C to the GLCD controller
'Clear GLCD content (text & graph), GLCD call:
'Sub Lcd_clear(byval Color As Word)
'Input:  wrdColor Color number to be set

   bytGD_Cmd = 2                                  'Clear command, clear all
   bytGD_Len = 4                                  '4 bytes length message
   bytHi = High(wrdColor)                         'High byte
   bytLo = Low(wrdColor)                          'Low byte

   Call IsLCDBusy                                 'Wait if LCD is still busy
   'Send it via I2C
   I2CStart
      I2CwByte bytGD_Addr                         'Write I2C address
      I2CwByte bytGD_Len                          '1: Message length
      I2CwByte bytGD_Cmd                          '2: Display command
      I2CwByte bytLo                              '3: color Lo byte
      I2CwByte bytHi                              '4: color Hi byte
   I2Cstop

End Sub

'==============================================================================
Sub GLCD_Text(byval strText As String , Byval wrdPosX As Word , Byval wrdPosY As Word , Byval bytFont As Byte , Byval wrdForecolor As Word , Byval wrdBackcolor As Word , Byval bytBold As Byte)
'Send the draw Text command via I2C to the GLCD controller
'GCLD call:
'Sub Lcd_text(byval Text As String , Byval Lcd_posx As Word , Byval Lcd_posy As Word , Byval Fontset As Byte , Byval Forecolor As Word , Byval Backcolor As Word , Byval Bold As Byte)
'Input:  wrdPosX        LCD start position X (horizontal)
'                       For landscape 0...319
'        wrdPosY        LCD start position Y (vertical)
'                       For landscape 0...239
'        bytFont        Font number
'        wrdForecolor   Text color
'        wrdBackcolor   Background color
'        bytBold        bold (1=normal size, 2...4: expanded by factor 2...4

   'Just for test ********************************
   Call ClearLCDLine(2)
   LCD strText
   '**********************************************

   bytGD_Cmd = 10                                 'Print string command

   bytLen = Len(strText)                          'Length of string
   If bytLen > TextLen Then bytLen = TextLen
   bytGD_Len = bytLen + 12                        'Total message length in bytes

   Call IsLCDBusy                                 'Wait if LCD is still busy
   'Send it via I2C
   I2CStart

      I2CwByte bytGD_Addr                         'Write I2C address
      I2CwByte bytGD_Len                          '1: Message length
      I2CwByte bytGD_Cmd                          '2: Print text command

      bytLo = Low(wrdPosX)
      I2CwByte bytLo                              '3: PosX lo byte
      bytHi = High(wrdPosX)
      I2CwByte bytHi                              '4: PosX hi byte

      bytLo = Low(wrdPosY)
      I2CwByte bytLo                              '5: PosY lo byte
      bytHi = High(wrdPosY)
      I2CwByte bytHi                              '64: PosY hi byte

      I2CwByte bytFont                            '7: Font

      bytLo = Low(wrdForecolor)
      I2CwByte bytLo                              '8: Forecolor lo byte
      bytHi = High(wrdForecolor)
      I2CwByte bytHi                              '9: Forecolor hi byte

      bytLo = Low(wrdBackcolor)
      I2CwByte bytLo                              '10: Backcolor lo byte
      bytHi = High(wrdBackcolor)
      I2CwByte bytHi                              '11: Backcolor hi byte

      I2CwByte bytBold                            '12: Backcolor lo byte

      For bytTmp0 = 1 To bytLen                   '13...: text data
         strChar1 = Mid(strText , bytTmp0 , 1)    'single character
         bytTmp1 = Asc(strChar1)                  'ASCII code of character
         I2CwByte bytTmp1                         'Write ASCII code
      Next bytTmp0
   I2Cstop

End Sub

'==============================================================================
Sub GLCD_Box(byval wrdPosX As Word , Byval wrdPosY As Word , Byval wrdLenX As Word , Byval wrdLenY As Word , Byval bytFill As Byte , Byval wrdBackColor As Word , Byval wrdBordercolor As Word)
'Send the draw Text command via I2C to the GLCD controller
'GCLD call:
'Call Lcd_Box(x start, y start, width, height, fill, color, bordercolor)
'Input:  wrdPosX        LCD start position X (horizontal)
'                       For landscape 0...319
'        wrdPosY        LCD start position Y (vertical)
'                       For landscape 0...239
'        wrdLenX        Length of box in x direction (horizontal)
'        wrdLenY        Height of box in y direction (vertical)
'        bytFill        1: Fill with back color
'                       2: Fill with border color
'                       3: Do not fill
'        wrdBackcolor   Background color
'        wrdBordercolor Border color

   bytGD_Cmd = 20                                 'Print Box command
   bytGD_Len = 15

   Call IsLCDBusy                                 'Wait if LCD is still busy
   'Send it via I2C
   I2CStart
      I2CwByte bytGD_Addr                         'Write I2C address
      I2CwByte bytGD_Len                          '1: Message length
      I2CwByte bytGD_Cmd                          '2: Print box command

      bytLo = Low(wrdPosX)
      I2CwByte bytLo                              '3: PosX lo byte
      bytHi = High(wrdPosX)
      I2CwByte bytHi                              '4: PosX hi byte

      bytLo = Low(wrdPosY)
      I2CwByte bytLo                              '5: PosY lo byte
      bytHi = High(wrdPosY)
      I2CwByte bytHi                              '6: PosY hi byte

      bytLo = Low(wrdLenX)
      I2CwByte bytLo                              '7: LenX lo byte
      bytHi = High(wrdLenX)
      I2CwByte bytHi                              '8: LenX hi byte

      bytLo = Low(wrdLenY)
      I2CwByte bytLo                              '9: LenY lo byte
      bytHi = High(wrdLenY)
      I2CwByte bytHi                              '10: LenY hi byte

      I2CwByte bytFill                            '11: bytFill

      bytLo = Low(wrdBackColor)
      I2CwByte bytLo                              '12: Back color lo byte
      bytHi = High(wrdBackColor)
      I2CwByte bytHi                              '13: Back color hi byte

      bytLo = Low(wrdBordercolor)
      I2CwByte bytLo                              '14: Border color lo byte
      bytHi = High(wrdBordercolor)
      I2CwByte bytHi                              '15: Border color hi byte
   I2CStop

End Sub

'==============================================================================
Sub GLCD_circle(byVal wrdPosX As Word , byVal wrdPosY As Word , byVal wrdRadius As Word , byVal wrdBordercolor As Word)
'Send the draw circle command via I2C to the GLCD controller
'GCLD call:
'Sub Lcd_circle(byval X As Word , Byval Y As Word , Byval Radius As Word , Byval Color As Word)
'Input:  wrdPosX        LCD circle center position X (horizontal)
'                       For landscape 0...319
'        wrdPosY        LCD circle center position Y (vertical)
'                       For landscape 0...239
'        wrdRadius      Circle radius
'        wrdBordercolor Circle Border color

   bytGD_Cmd = 30                                 'Print Box command
   bytGD_Len = 10

   Call IsLCDBusy                                 'Wait if LCD is still busy
   'Send it via I2C
   I2CStart
      I2CwByte bytGD_Addr                         'Write I2C address
      I2CwByte bytGD_Len                          '1: Message length
      I2CwByte bytGD_Cmd                          '2: Print box command

      bytLo = Low(wrdPosX)
      I2CwByte bytLo                              '3: PosX lo byte
      bytHi = High(wrdPosX)
      I2CwByte bytHi                              '4: PosX hi byte

      bytLo = Low(wrdPosY)
      I2CwByte bytLo                              '5: PosY lo byte
      bytHi = High(wrdPosY)
      I2CwByte bytHi                              '6: PosY hi byte

      bytLo = Low(wrdRadius)
      I2CwByte bytLo                              '7: Radius lo byte
      bytHi = High(wrdRadius)
      I2CwByte bytHi                              '8: Radius hi byte

      bytLo = Low(wrdBordercolor)
      I2CwByte bytLo                              '9: Border color lo byte
      bytHi = High(wrdBordercolor)
      I2CwByte bytHi                              '10: Border color hi byte
   I2CStop

End Sub

'==============================================================================
Sub GLCD_fill_circle(byVal wrdPosX As Word , byVal wrdPosY As Word , byVal wrdRadius As Word , byVal wrdBackcolor As Word)
'Send the draw filled circle command via I2C to the GLCD controller
'GCLD call:
'Sub Lcd_circle(byval X As Word , Byval Y As Word , Byval Radius As Word , Byval Color As Word)
'Input:  wrdPosX        LCD circle center position X (horizontal)
'                       For landscape 0...319
'        wrdPosY        LCD circle center position Y (vertical)
'                       For landscape 0...239
'        wrdRadius      Circle radius
'        wrdBackcolor   Circle full color

   bytGD_Cmd = 32                                 'Print Box command
   bytGD_Len = 10

   Call IsLCDBusy                                 'Wait if LCD is still busy
   'Send it via I2C
   I2CStart
      I2CwByte bytGD_Addr                         'Write I2C address
      I2CwByte bytGD_Len                          '1: Message length
      I2CwByte bytGD_Cmd                          '2: Print box command

      bytLo = Low(wrdPosX)
      I2CwByte bytLo                              '3: PosX lo byte
      bytHi = High(wrdPosX)
      I2CwByte bytHi                              '4: PosX hi byte

      bytLo = Low(wrdPosY)
      I2CwByte bytLo                              '5: PosY lo byte
      bytHi = High(wrdPosY)
      I2CwByte bytHi                              '6: PosY hi byte

      bytLo = Low(wrdRadius)
      I2CwByte bytLo                              '7: Radius lo byte
      bytHi = High(wrdRadius)
      I2CwByte bytHi                              '8: Radius hi byte

      bytLo = Low(wrdBackcolor)
      I2CwByte bytLo                              '9: Back color lo byte
      bytHi = High(wrdBackcolor)
      I2CwByte bytHi                              '10: Back color hi byte
   I2CStop

End Sub

'==============================================================================
Sub GLCD_line(byVal wrdPosX1 As Word , byVal wrdPosY1 As Word , byVal wrdPosX2 As Word , byVal wrdPosY2 As Word , byVal bytWidth As Byte , byVal wrdLineColor As Word)
'Send the draw line command via I2C to the GLCD controller
'GCLD call:
'Sub Lcd_line(byval X1 As Word , Byval Y1 As Word , Byval X2 As Word , Byval Y2 As Word , Byval Pen_width As Byte , Byval Color As Word)
'Input:  wrdPosX1       Line start position X (horizontal)
'                       For landscape 0...319
'        wrdPosY1       Line start position Y (vertical)
'                       For landscape 0...239
'        wrdPosX2       Line end position X (vertical column)
'        wrdPosY2       Line end position Y (horizontal row)
'        bytWidth       Line width (pixel)
'        wrdLinecolor   Line color

   bytGD_Cmd = 40                                 'Print line command
   bytGD_Len = 13

   Call IsLCDBusy                                 'Wait if LCD is still busy
   'Send it via I2C
   I2CStart
      I2CwByte bytGD_Addr                         'Write I2C address
      I2CwByte bytGD_Len                          '1: Message length
      I2CwByte bytGD_Cmd                          '2: Print box command

      bytLo = Low(wrdPosX1)
      I2CwByte bytLo                              '3: PosX1 lo byte
      bytHi = High(wrdPosX1)
      I2CwByte bytHi                              '4: PosX1 hi byte

      bytLo = Low(wrdPosY1)
      I2CwByte bytLo                              '5: PosY1 lo byte
      bytHi = High(wrdPosY1)
      I2CwByte bytHi                              '6: PosY1 hi byte

      bytLo = Low(wrdPosX2)
      I2CwByte bytLo                              '7: PosX2 lo byte
      bytHi = High(wrdPosX2)
      I2CwByte bytHi                              '8: PosX2 hi byte

      bytLo = Low(wrdPosY2)
      I2CwByte bytLo                              '9: PosY2 lo byte
      bytHi = High(wrdPosY2)
      I2CwByte bytHi                              '10: PosY2 hi by

      I2CwByte bytWidth                           '11: Line width

      bytLo = Low(wrdLineColor)
      I2CwByte bytLo                              '12: Line color lo byte
      bytHi = High(wrdLineColor)
      I2CwByte bytHi                              '13: Line color hi byte
   I2CStop

End Sub

End